"""
Configs migration

Generation datetime: 2024-07-22 06:38:26.289110
"""

import ujson
from configs_core.migration import models
from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)
from sqlalchemy import and_, insert, outerjoin, select, update

mh: DatabaseMigrationHelper

REVISION = "46b1afe4"
DOWN_REVISION = "6380e74e"

limitationsToModify: list[LimitationToModify] = []

externalLimitationsToModify: list[ExternalLimitationToModify] = []


def getSetting(name: str) -> list:
    """
    Get setting by name from db.
    Args:
        name: setting name
    Returns:
        settings list
    """
    dbReply = mh.connection.execute(
        select(
            [models.Tag.tag, models.Setting.value, models.Setting.description],
            from_obj=outerjoin(models.Setting, models.Tag, models.Setting.id == models.Tag.id),
        ).where(models.Setting.name == name)
    ).cursor.fetchall()
    return [(tag, ujson.loads(value), description) for tag, value, description in dbReply]


def getLimitation(name: str) -> dict:
    """
    Get limitation by name from db.
    Args:
        name: limitation name
    Returns:
        limitation as dict
    """
    record = mh.connection.execute(
        select(
            [
                models.Limitation.validation_schema,
                models.Limitation.services,
                models.Limitation.default_value,
                models.Limitation.description,
            ]
        ).where(models.Limitation.limitation_name == name)
    ).cursor.fetchall()[0]
    limitation = {
        "validation_schema": ujson.loads(record[0]),
        "services": record[1].split(","),
        "default_value": ujson.loads(record[2]),
        "description": record[3],
    }
    return limitation


def getGroups(name: str) -> list[str]:
    """
    Get limitation groups from db.
    Args:
        name: setting name
    Returns:
        list of groups
    """
    records = mh.connection.execute(
        select([models.GroupLimitation.group_name]).where(models.GroupLimitation.limitation_name == name)
    ).cursor.fetchall()
    groups = [row[0] for row in records]
    return groups


def setSetting(value: dict, name: str, description: str, tag: str | None = None):
    """
    Set setting by name.
    Args:
        value: settings to be set
        name: luna-handlers setting name
        description: settings description
        tag: setting tag
    """
    if tag is None:
        mh.connection.execute(
            update(models.Setting)
            .where(and_(models.Setting.name == name, models.Setting.is_default == True))
            .values(value=ujson.dumps(value), description=description)
        )
    else:
        settingQuery = insert(models.Setting).values(  # type: ignore
            value=ujson.dumps(value),
            name=name,
            is_default=False,
            description=description,
        )
        record = mh.connection.execute(settingQuery)
        tagQuery = insert(models.Tag).values(id=record.inserted_primary_key[0], name=name, tag=tag)
        mh.connection.execute(tagQuery)


def upgrade():
    """Upgrade configs to new revision"""

    monitoringSettings = getSetting("INFLUX_MONITORING")
    monitoringLimitation = getLimitation("INFLUX_MONITORING")
    monitoringGroups = getGroups("INFLUX_MONITORING")
    # limitations to remove
    mh.removeLimitation(limitationName="INFLUX_MONITORING")

    validationSchema = monitoringLimitation["validation_schema"]
    validationSchema["properties"]["storage_type"] = {"type": "string", "enum": ["influx"]}
    validationSchema["default"]["storage_type"] = "influx"
    validationSchema["required"].insert(0, "storage_type")
    validationSchema["description"] = "Influx monitoring settings."

    monitoringLimitation["default_value"]["storage_type"] = "influx"

    # limitations to add
    mh.addLimitation(
        limitationName="LUNA_MONITORING",
        validationSchema={"oneOf": [validationSchema]},
        defaultValue=monitoringLimitation["default_value"],
        description=monitoringLimitation["description"],
        services=monitoringLimitation["services"],
        groupNames=monitoringGroups,
    )

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    for tag, value, description in monitoringSettings:
        value["storage_type"] = "influx"
        setSetting(value=value, name="LUNA_MONITORING", tag=tag, description=description)


def downgrade():
    """Downgrade configs to previous revision"""

    monitoringSettings = getSetting("LUNA_MONITORING")
    monitoringLimitation = getLimitation("LUNA_MONITORING")
    monitoringGroups = getGroups("LUNA_MONITORING")

    validationSchema = monitoringLimitation["validation_schema"]["oneOf"][0]
    validationSchema["properties"].pop("storage_type")
    validationSchema["default"].pop("storage_type")
    validationSchema["required"].remove("storage_type")
    validationSchema["description"] = "Monitoring settings."

    monitoringLimitation["default_value"].pop("storage_type")

    # limitations to modify
    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to add
    mh.addLimitation(
        limitationName="INFLUX_MONITORING",
        validationSchema=validationSchema,
        defaultValue=monitoringLimitation["default_value"],
        description=monitoringLimitation["description"],
        services=monitoringLimitation["services"],
        groupNames=monitoringGroups,
    )

    # limitations to remove
    mh.removeLimitation(limitationName="LUNA_MONITORING")

    for tag, value, description in monitoringSettings:
        value.pop("storage_type")
        setSetting(value=value, name="INFLUX_MONITORING", tag=tag, description=description)
