"""
Module contains settings class for luna-tasks
"""

from .configs.configs.services.base_python_service_settings import BasePythonServiceSettingsClass
from .configs.configs.settings.all_settings import SETTINGS
from .configs.configs.settings.classes import (
    AdditionalServicesUsageSetting as _AdditionalServicesUsageSetting,
    DBSetting as _DBSetting,
    ImageStoreAddressSettings as _ImageStoreAddressSettings,
    PlatformLimitsSettings as _PlatformLimitsSettings,
    RedisStorageSetting as _RedisStorageSetting,
    ServiceAddressSettings as _ServiceAddressSettings,
    ServiceTimeoutsSettings as _ServiceTimeoutsSettings,
)


class ServiceSettings(BasePythonServiceSettingsClass):
    """
    Settings class for luna-lambda-tasks-unit
    Attributes:
        db(DBSetting): database settings
        faceSamplesStorage(ImageStoreAddressSettings): face samples storage settings
        bodySamplesStorage(ImageStoreAddressSettings): body samples storage settings
        imageOriginStorage(ImageStoreAddressSettings): image origin storage settings
        objectsOriginStorage(ImageStoreAddressSettings): objects origin storage settings
        faceSamplesStorageTimeouts(ServiceTimeoutsSettings): Luna Image Store client timeouts for face samples
        bodySamplesStorageTimeouts(ServiceTimeoutsSettings): Luna Image Store client timeouts for body samples
        taskResultStorage(ImageStoreAddressSettings): Luna Image Store address for task results
        taskResultStorageTimeouts(ServiceTimeoutsSettings): Luna Image Store client timeouts for task results
        taskExternalArchiveTimeouts(ServiceTimeoutsSettings): Luna Task load external archive timeouts
        facesAddress(ServiceAddressSettings): Luna Faces address
        facesTimeouts(ServiceTimeoutsSettings): Luna Faces timeouts
        eventsAddress(ServiceAddressSettings): Luna Events address
        eventsTimeouts(ServiceTimeoutsSettings): Luna Events timeouts
        additionalServicesUsage(SupportServicesSetting): support services container
        storageTime(str): time for records in the database
        maxErrorCountPerTask(int): maximum errors count per task to store in the database
        tasksToImageStoreRequestsConcurrency(int): concurrency of requests to Luna Image Store
        tasksToFacesRequestsConcurrency(int): concurrency of requests to Luna Faces
        tasksToMatcherConcurrency(int): concurrency of requests to Luna Python Matcher
        pythonMatcherAddress(ServiceAddressSettings): luna-python-matcher address settings
        matcherProxyAddress(ServiceAddressSettings): python-matcher-proxy address settings
        matcherProxyTimeouts(ServiceAddressSettings): luna-python-matcher address settings
        tasksToMatcherTimeouts(ServiceTimeoutsSettings): timeouts for luna-python-matcher and matcher-proxy settings
        platformLimits (PlatformLimitsSettings): platform limits
        externalApiAddress (_ServiceAddressSettings): external luna api address
        redisDB (RedisStorageSetting): redis db address
        lambdaTasksUnitTimeouts (_ServiceTimeoutsSettings): luna-lambda-tasks timeouts settings
        remoteSDKAddress(_ServiceAddressSettings): external luna remote sdk address
        remoteSDKTimeouts(ServiceTimeoutsSettings): luna-remote-sdk timeouts settings
        videoAgentAddress(_ServiceAddressSettings): external luna video agent address
        videoAgentTimeouts(ServiceTimeoutsSettings): luna-video-agent timeouts settings
        senderAddress (ServiceAddressSettings): luna-sender address settings
        pythonMatcherTimeouts(ServiceTimeoutsSettings): luna-python-matcher timeouts settings
    """

    serviceName = "luna-lambda-tasks-unit"
    settingsStore = SETTINGS

    def __init__(self):
        super().__init__()
        self.db: _DBSetting | None = None
        self.faceSamplesStorage: _ImageStoreAddressSettings | None = None
        self.bodySamplesStorage: _ImageStoreAddressSettings | None = None
        self.imageOriginStorage: _ImageStoreAddressSettings | None = None
        self.faceSamplesStorageTimeouts: _ServiceTimeoutsSettings | None = None
        self.bodySamplesStorageTimeouts: _ServiceTimeoutsSettings | None = None
        self.imageOriginStorageTimeouts: _ServiceTimeoutsSettings | None = None
        self.taskResultStorage: _ImageStoreAddressSettings | None = None
        self.taskResultStorageTimeouts: _ServiceTimeoutsSettings | None = None
        self.taskExternalArchiveTimeouts: _ServiceTimeoutsSettings | None = None
        self.facesAddress: _ServiceAddressSettings | None = None
        self.facesTimeouts: _ServiceTimeoutsSettings | None = None
        self.eventsAddress: _ServiceAddressSettings | None = None
        self.eventsTimeouts: _ServiceTimeoutsSettings | None = None
        self.platformLimits: _PlatformLimitsSettings | None = None

        # LUNA TASKS ROUTER SETTINGS
        self.additionalServicesUsage: _AdditionalServicesUsageSetting | None = None
        self.storageTime: str | None = None

        # LUNA TASKS WORKER SETTINGS
        self.maxErrorCountPerTask: int | None = None
        self.tasksToImageStoreRequestsConcurrency: int | None = None
        self.tasksToFacesRequestsConcurrency: int | None = None
        self.handlersAddress: _ServiceAddressSettings | None = None
        self.handlersTimeouts: _ServiceTimeoutsSettings | None = None
        self.tasksToHandlersConcurrency: int | None = None
        self.tasksToMatcherConcurrency: int | None = None
        self.objectsOriginStorage: _ImageStoreAddressSettings | None = None

        self.pythonMatcherAddress: _ServiceAddressSettings | None = None
        self.pythonMatcherTimeouts: _ServiceTimeoutsSettings | None = None
        self.matcherProxyAddress: _ServiceAddressSettings | None = None
        self.matcherProxyTimeouts: _ServiceTimeoutsSettings | None = None
        self.tasksToMatcherTimeouts: _ServiceTimeoutsSettings | None = None
        self.externalApiAddress: _ServiceAddressSettings | None = None
        self.redisDB: _RedisStorageSetting | None = None
        self.lambdaTasksUnitTimeouts: _ServiceTimeoutsSettings | None = None
        self.remoteSDKAddress: _ServiceAddressSettings | None = None
        self.remoteSDKTimeouts: _ServiceTimeoutsSettings | None = None
        self.videoAgentAddress: _ServiceAddressSettings | None = None
        self.videoAgentTimeouts: _ServiceTimeoutsSettings | None = None
        self.senderAddress: _ServiceAddressSettings | None = None

    def applySettings(self):
        """
        Deserialize loaded settings to classes
        """
        super().applySettings()
        self.db = _DBSetting(self.serviceName, self["LUNA_TASKS_DB"])
        self.faceSamplesStorage = _ImageStoreAddressSettings(
            self.serviceName, "LUNA_IMAGE_STORE_FACES_SAMPLES_ADDRESS", self["LUNA_IMAGE_STORE_FACES_SAMPLES_ADDRESS"]
        )
        self.bodySamplesStorage = _ImageStoreAddressSettings(
            self.serviceName, "LUNA_IMAGE_STORE_BODIES_SAMPLES_ADDRESS", self["LUNA_IMAGE_STORE_BODIES_SAMPLES_ADDRESS"]
        )
        self.imageOriginStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_IMAGES_ADDRESS",
            self["LUNA_IMAGE_STORE_IMAGES_ADDRESS"],
        )
        self.faceSamplesStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_IMAGE_STORE_FACES_SAMPLES_TIMEOUTS", self["LUNA_IMAGE_STORE_FACES_SAMPLES_TIMEOUTS"]
        )
        self.bodySamplesStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_BODIES_SAMPLES_TIMEOUTS",
            self["LUNA_IMAGE_STORE_BODIES_SAMPLES_TIMEOUTS"],
        )
        self.imageOriginStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_IMAGES_TIMEOUTS",
            self["LUNA_IMAGE_STORE_IMAGES_TIMEOUTS"],
        )
        self.taskResultStorage = _ImageStoreAddressSettings(
            self.serviceName, "LUNA_IMAGE_STORE_TASK_RESULT_ADDRESS", self["LUNA_IMAGE_STORE_TASK_RESULT_ADDRESS"]
        )
        self.taskResultStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_IMAGE_STORE_TASK_RESULT_TIMEOUTS", self["LUNA_IMAGE_STORE_TASK_RESULT_TIMEOUTS"]
        )
        self.facesAddress = _ServiceAddressSettings(self.serviceName, "LUNA_FACES_ADDRESS", self["LUNA_FACES_ADDRESS"])
        self.facesTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_FACES_TIMEOUTS", self["LUNA_FACES_TIMEOUTS"]
        )
        self.eventsAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_EVENTS_ADDRESS", self["LUNA_EVENTS_ADDRESS"]
        )
        self.eventsTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_EVENTS_TIMEOUTS", self["LUNA_EVENTS_TIMEOUTS"]
        )
        self.taskExternalArchiveTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_TASKS_LOAD_EXTERNAL_ARCHIVE_TIMEOUTS",
            self["LUNA_TASKS_LOAD_EXTERNAL_ARCHIVE_TIMEOUTS"],
        )
        self.platformLimits = _PlatformLimitsSettings(self.serviceName, self["PLATFORM_LIMITS"])

        # LUNA TASKS ROUTER SETTINGS
        self.additionalServicesUsage = _AdditionalServicesUsageSetting(
            self.serviceName, self["ADDITIONAL_SERVICES_USAGE"]
        )

        # LUNA TASKS WORKER SETTINGS
        self.maxErrorCountPerTask = self["max_error_count_per_task"]
        self.tasksToImageStoreRequestsConcurrency = self["tasks_to_image_store_requests_concurrency"]
        self.tasksToFacesRequestsConcurrency = self["tasks_to_faces_requests_concurrency"]
        self.handlersAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_HANDLERS_ADDRESS", self["LUNA_HANDLERS_ADDRESS"]
        )
        self.handlersTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_HANDLERS_TIMEOUTS", self["LUNA_HANDLERS_TIMEOUTS"]
        )
        self.tasksToHandlersConcurrency = self["TASKS_TO_HANDLERS_CONCURRENCY"]

        self.tasksToMatcherConcurrency = self["TASKS_TO_MATCHER_CONCURRENCY"]

        self.pythonMatcherAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_PYTHON_MATCHER_ADDRESS", self["LUNA_PYTHON_MATCHER_ADDRESS"]
        )
        self.pythonMatcherTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_PYTHON_MATCHER_TIMEOUTS",
            self["LUNA_PYTHON_MATCHER_TIMEOUTS"],
        )
        self.matcherProxyAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_MATCHER_PROXY_ADDRESS", self["LUNA_MATCHER_PROXY_ADDRESS"]
        )
        self.matcherProxyTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_PYTHON_MATCHER_PROXY_TIMEOUTS", self["LUNA_PYTHON_MATCHER_PROXY_TIMEOUTS"]
        )
        self.tasksToMatcherTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_TASKS_TO_MATCHER_TIMEOUTS", self["LUNA_TASKS_TO_MATCHER_TIMEOUTS"]
        )
        self.externalApiAddress = _ServiceAddressSettings(
            self.serviceName,
            "EXTERNAL_LUNA_API_ADDRESS",
            self["EXTERNAL_LUNA_API_ADDRESS"],
        )
        self.objectsOriginStorage = _ImageStoreAddressSettings(
            self.serviceName, "LUNA_IMAGE_STORE_OBJECTS_ADDRESS", self["LUNA_IMAGE_STORE_OBJECTS_ADDRESS"]
        )
        self.redisDB = _RedisStorageSetting(self.serviceName, "TASKS_REDIS_DB_ADDRESS", self["TASKS_REDIS_DB_ADDRESS"])
        self.remoteSDKAddress = _ServiceAddressSettings(
            self.serviceName,
            "LUNA_REMOTE_SDK_ADDRESS",
            self["LUNA_REMOTE_SDK_ADDRESS"],
        )
        self.remoteSDKTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_REMOTE_SDK_TIMEOUTS",
            self["LUNA_REMOTE_SDK_TIMEOUTS"],
        )
        self.videoAgentAddress = _ServiceAddressSettings(
            self.serviceName,
            "LUNA_VIDEO_AGENT_ADDRESS",
            self["LUNA_VIDEO_AGENT_ADDRESS"],
        )
        self.videoAgentTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_VIDEO_AGENT_TIMEOUTS",
            self["LUNA_VIDEO_AGENT_TIMEOUTS"],
        )
        self.senderAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_SENDER_ADDRESS", self["LUNA_SENDER_ADDRESS"]
        )

        return self
